/* $Id: vircam_jmp_utils.h,v 1.16 2012-01-16 12:32:18 jim Exp $
 *
 * This file is part of the VIRCAM Pipeline
 * Copyright (C) 2015 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: jim $
 * $Date: 2012-01-16 12:32:18 $
 * $Revision: 1.16 $
 * $Name: not supported by cvs2svn $
 */


/* Includes */

#ifndef VIRCAM_JMP_UTILS_H
#define VIRCAM_JMP_UTILS_H

#include <cpl.h>

#define INITALLOC 16


struct vircam_jmp_configStruct {

    /* Input */

    int         ipix;
    float       threshold;
    int         icrowd;
    float       rcore;
    int         nbsize;
    int         savecat;
    int         destripe;
    int         skycor;
    int         nord;
    int         savesimple;
    int         extenum;

};

typedef struct {
    int          ustep_number;
    int          nframes;
    int          ngood;
    int          nustep;
    int          status;
    casu_fits    **f;
    casu_fits    *super;
    casu_fits    *superc;
} ustep_set;
    
struct psStruct {

    /* Level 0 stuff */

    cpl_size         *labels;
    cpl_frame        *master_dark;
    cpl_frame        *master_twilight_flat;
    cpl_frame        *master_conf;
    cpl_frame        *master_sky;
    casu_mask         *mask;
    cpl_frame        *chantab;
    cpl_frame        *phottab;
    cpl_table        *tphottab;
    cpl_frame        *readgain_file;
    cpl_frameset     *science_frames;
    cpl_frameset     *offset_skies;
    cpl_frame        *product_frame_stack;
    cpl_frame        *product_frame_stackc;
    cpl_frame        *product_frame_cat;
    cpl_frame        *product_frame_illcor;
    cpl_frame        *product_frame_sky;
    cpl_frame        **product_frames_simple;
    cpl_frame        **product_frames_simple_off;
    cpl_frame        **product_frames_super;
    cpl_frame        **product_frames_superc;
    cpl_propertylist *phupaf;
    float            *gaincors;
    char             *catpath;
    char             *catname;
    char             *catpath2;
    char             *catname2;

    /* Level 1 stuff */

    casu_fits        *fdark;
    casu_fits        *fflat;
    casu_fits        *fconf;
    casu_fits        *fsky;
    casu_tfits       *fchantab;

    int              nscience;
    casu_fits        **sci_fits;

    int              noffsets;
    casu_fits        **offsky_fits;

    int              nustep_sets;
    ustep_set        *ustep_sets;

    int              ndith;
    int              ndithc;
    casu_fits        **dith_input;
    casu_fits        **dithc_input;

    casu_fits        *stack_frame;
    casu_fits        *stackc_frame;
    casu_tfits       *outcat;

    casu_fits        *outsky;

    cpl_table        *matchstds;

    casu_tfits       *illcor;

};


int isfirst;
int interlv;
char current_cat[VIRCAM_PATHSZ];
char current_catpath[VIRCAM_PATHSZ];
char vircam_recipename[VIRCAM_PATHSZ];
char vircam_recipepaf[VIRCAM_PATHSZ];
cpl_propertylist *dummyqc = NULL;
int recflag;



extern int vircam_jmp_save_simple(struct psStruct* ps, cpl_frameset *framelist, 
				  cpl_parameterlist *parlist);
extern int vircam_jmp_save_simple_offsets(struct psStruct* ps, cpl_frameset *framelist, 
					  cpl_parameterlist *parlist);
extern int vircam_jmp_save_offsky(struct psStruct* ps, cpl_frameset *framelist, 
				  cpl_parameterlist *parlist);
extern int vircam_jmp_save_super(struct psStruct* ps, cpl_frameset *framelist, 
				 cpl_parameterlist *parlist);
extern int vircam_jmp_save_stack(struct psStruct* ps, cpl_frameset *framelist, 
				 cpl_parameterlist *parlist, cpl_propertylist *dummyqcprops);
extern int vircam_jmp_save_catalogue(struct psStruct* ps, cpl_frameset *framelist, 
				     cpl_parameterlist *parlist, cpl_propertylist *dummyqcprops);
extern int vircam_jmp_save_illum(struct psStruct* ps, cpl_frameset *framelist, 
				 cpl_parameterlist *parlist, cpl_propertylist *dummyqcprops);
extern void vircam_jmp_ustep_seq      (struct psStruct* ps);
extern void vircam_jmp_interleave     (struct psStruct* ps);
extern void vircam_jmp_dither_offsets (struct psStruct* ps, int inter_lv);
extern void vircam_jmp_dither_images  (struct psStruct* ps);
extern void vircam_jmp_catalogue      (struct psStruct* ps, int inter_lv);
extern void vircam_jmp_matched_stds   (struct psStruct* ps, char* cat, char* catpath);
extern void vircam_jmp_wcsfit         (struct psStruct* ps);
extern void vircam_jmp_photcal        (struct psStruct* ps);
extern void vircam_jmp_bpm2conf       (struct psStruct* ps);
extern void vircam_jmp_skycor         (struct psStruct* ps);
extern void vircam_jmp_get_readnoise_gain(struct psStruct* ps, int jext, float *readnoise, 
					  float *gain);
extern void vircam_jmp_illum          (struct psStruct* ps, char* cat, char* catpath);
extern void vircam_jmp_init           (struct psStruct* ps);
extern void vircam_jmp_tidy           (struct psStruct* ps, int level);



void vircam_jmp_set_config_ipix       (int   ipix)        ; 
void vircam_jmp_set_config_icrowd     (int   icrowd)      ;
void vircam_jmp_set_config_rcore      (float rcore)       ;
void vircam_jmp_set_config_threshold  (float threshold)   ;
void vircam_jmp_set_config_nbsize     (int   nbsize)      ;
void vircam_jmp_set_config_savecat    (int   savecat)     ;	      
void vircam_jmp_set_config_destripe   (int   destripe)    ;
void vircam_jmp_set_config_skycor     (int   skycore)     ;
void vircam_jmp_set_config_savesimple (int   savesimple)  ;
void vircam_jmp_set_config_extenum    (int   extenum)     ;
void vircam_jmp_set_offsky(int offsky);
void vircam_jmp_set_isfirst(int isfirst);
void vircam_set_recipename(const char* recipename,int recipe_flag);

int vircam_jmp_get_config_savecat    ();	     
int vircam_jmp_get_config_destripe   ();
int vircam_jmp_get_config_skycor     ();
int vircam_jmp_get_config_savesimple ();
int vircam_jmp_get_config_extenum    ();


/* Define the flags for which recipe these routines are servicing... */

# define RECSCI 1
# define RECSTD 2
# define RECMES 3

#endif

/*

$Log: not supported by cvs2svn $
Revision 1.15  2010/09/10 11:24:06  jim
Added master_sky and fsky to ps structure

Revision 1.14  2010/01/31 19:27:32  jim
Added global variable to allow for simple products to be optionally saved

Revision 1.13  2009/09/22 12:28:26  jim
Added offset sky items to ps structure. Added routines
vircam_jmp_save_simple_offsets and vircam_jmp_save_offset

Revision 1.12  2008/11/25 18:55:16  jim
took out extra sky stuff

Revision 1.11  2008/11/25 11:57:30  jim
skycor variable is now a string in the config structure. A list of sky
algorithms also included.

Revision 1.10  2007/10/19 06:55:06  jim
Modifications made to use new method for directing the recipes to the
standard catalogues using the sof

Revision 1.9  2007/05/08 10:41:50  jim
Added gaincor variables

Revision 1.8  2007/04/04 16:05:59  jim
Modified to make paf information a bit more correct

Revision 1.7  2007/04/04 10:34:55  jim
Modified to use new dfs tags

Revision 1.6  2007/03/14 14:49:13  jim
Fixed problem with missing paf files in jmp recipes if detlive = F. Also
fixed problem where extra dummy products were being created

Revision 1.5  2007/03/01 12:42:42  jim
Modified slightly after code checking

Revision 1.4  2006/12/04 21:08:26  jim
Added nord to vircam_jmp_config

Revision 1.3  2006/11/29 12:28:45  jim
Modified so that the correct recipe names would appear in the headers of
data products

Revision 1.2  2006/11/28 20:55:10  jim
Added illcor to ps structure

Revision 1.1  2006/11/27 11:54:37  jim
Initial entry


*/
